create or replace procedure temp_recompile_plsql (
  exclusion_list in varchar
)
as
  success_with_compilation_error exception;
  PRAGMA EXCEPTION_INIT(success_with_compilation_error,-24344);
begin

  for cur in (select object_name from user_objects where status != 'VALID' and object_type = 'PACKAGE') loop
    if exclusion_list not like '% ' || cur.object_name || ' %' then
      dbms_output.put_line('Rebuilding package specification for ' || cur.object_name);
      execute immediate 'alter package ' || cur.object_name || ' compile specification';
    end if;
  end loop;

  for cur in (select object_name from user_objects where status != 'VALID' and object_type = 'PACKAGE BODY') loop
    if exclusion_list not like '% ' || cur.object_name || ' %' then
      dbms_output.put_line('Rebuilding package body for ' || cur.object_name);
      execute immediate 'alter package ' || cur.object_name || ' compile body';
    end if;
  end loop;

  EXCEPTION
    when success_with_compilation_error then
      dbms_output.put_line('Compilation failed.');
end;
/
show errors

create or replace procedure temp_recompile_views
as 
begin
  for cur in (select object_name,text from user_objects o, user_views v where  o.status != 'VALID' and o.object_type = 'VIEW' and v.view_name = o.object_name) loop
    dbms_output.put_line('Rebuilding view ' || cur.object_name);
    execute immediate 'create or replace view ' || cur.object_name || ' as ' || cur.text;
  end loop;
end;
/
show errors;
